package com.mycompany.commerce.project.facade.server.commands;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandContextImpl;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.foundation.common.exception.AbstractApplicationException;
import com.ibm.commerce.foundation.common.exception.ApplicationError;
import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.foundation.internal.common.exception.FoundationApplicationException;
import com.ibm.commerce.foundation.server.command.bod.SearchExpression;
import com.ibm.commerce.foundation.server.command.bod.bom.AbstractChangeNounPartUsingBusinessObjectMediatorCmdImpl;
import com.ibm.commerce.sample.commands.ProjectUpdateCmdImpl;
import com.mycompany.commerce.project.facade.datatypes.ProjectType;
import com.mycompany.commerce.project.logging.ProjectApplicationMessageKeys;

/**
 * This command is used to change the base part of Project nouns.
 * The parent class will call the <code>ChangeProjectBasePartMediator</code>
 * to persist the changes to the database. 
 */
public class ChangeProjectBasePartCmdImpl extends AbstractChangeNounPartUsingBusinessObjectMediatorCmdImpl implements ChangeProjectPartActionCmd {


	/**
	 * Constants for logging and tracing.
	 */
	private static final String CLASSNAME = ChangeProjectBasePartCmdImpl.class.getName();
	private static final Logger LOGGER = LoggingHelper.getLogger(ChangeProjectBasePartCmdImpl.class);
	
	/**
	 * This method is the default constructor. It calls the parent method which will call the change mediator.
	 */
	public ChangeProjectBasePartCmdImpl() {
		super();
		final String METHODNAME = "ChangeProjectBasePartCmdImpl()";
    	if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
    		LOGGER.entering(CLASSNAME, METHODNAME);
    		LOGGER.exiting(CLASSNAME, METHODNAME);
    	}
	}
	
	/**
	 * This method performs custom business logic validation when adding a noun part to a noun.
	 * This implemtation blocks the request from proceeding.
	 * @param noun The noun to perform the operation on.
	 * @param nounPart The part of the noun to add.
	 * @param controlParameters The control parameters associated with the operation.
	 * @throws AbstractApplicationException A problem occurred while adding the noun part.
	 */
	protected void validateAdd(Object noun, Object nounPart, List controlParameters) throws AbstractApplicationException {
		final String METHODNAME = "validateAdd(Object,Object,List)";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] param = { noun, nounPart, controlParameters };
			LOGGER.entering(CLASSNAME, METHODNAME, param);
		}
		
		// TODO: Add business logic validation.
		
		// The following sample code creates an invalid operation exception. 	
		//List validationErrors = new ArrayList();
		//ApplicationError validateError = new ApplicationError(
		//		ApplicationError.TYPE_GENERIC_ERROR,
		//		ProjectApplicationMessageKeys._APP_INVALID_OPERATION,
		//		new Object[] {noun}, LOGGER.getResourceBundleName());
		//validationErrors.add(validateError);
		//if (validationErrors != null && validationErrors.size() > 0) {
		//	throw new FoundationApplicationException(validationErrors, CLASSNAME, METHODNAME, null);
		//}
		
		// Must always call super() to run the data service layer validation.
		super.validateAdd(noun, nounPart, controlParameters);						
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}
	/**
	 * This method performs custom business logic validation when changing a noun part.
	 * This implemtation blocks the request from proceeding.	 
	 * @param noun The noun to perform the operation on.
	 * @param nounPart The part of the noun to change.
	 * @param controlParameters The control parameters associated with the operation.
	 * @throws AbstractApplicationException A problem occurred while adding the noun part.
	 */
	protected void validateChange(Object noun, Object nounPart, List controlParameters) throws AbstractApplicationException {
		final String METHODNAME = "validateChange(Object,Object,List)";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] param = { noun, nounPart, controlParameters };
			LOGGER.entering(CLASSNAME, METHODNAME, param);
		}
		
		// TODO: Add business logic validation.
		
		// The following sample code creates an invalid operation exception. 	
		//List validationErrors = new ArrayList();
		//ApplicationError validateError = new ApplicationError(
		//		ApplicationError.TYPE_GENERIC_ERROR,
		//		ProjectApplicationMessageKeys._APP_INVALID_OPERATION,
		//		new Object[] {noun}, LOGGER.getResourceBundleName());
		//validationErrors.add(validateError);
		//if (validationErrors != null && validationErrors.size() > 0) {
		//	throw new FoundationApplicationException(validationErrors, CLASSNAME, METHODNAME, null);
		//}
		
		// Must always call super() to run the data service layer validation.
		super.validateChange(noun, nounPart, controlParameters);	
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}
	/**
	 * This method performs custom business logic validation when deleting a noun part.
	 * This implementation blocks the request from proceeding.	 
	 * @param noun The noun to perform the operation on.
	 * @param nounPart The part of the noun to delete.
	 * @param controlParameters The control parameters associated with the operation.
	 * @throws AbstractApplicationException A problem occurred while adding the noun part.
	 */
	protected void validateDelete(Object noun, Object nounPart, List controlParameters) throws AbstractApplicationException {
		final String METHODNAME = "validateDelete(Object,Object,List)";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			Object[] param = { noun, nounPart, controlParameters };
			LOGGER.entering(CLASSNAME, METHODNAME, param);
		}
		
		// TODO: Add business logic validation.
		
		// The following sample code creates an invalid operation exception. 	
		//List validationErrors = new ArrayList();
		//ApplicationError validateError = new ApplicationError(
		//		ApplicationError.TYPE_GENERIC_ERROR,
		//		ProjectApplicationMessageKeys._APP_INVALID_OPERATION,
		//		new Object[] {noun}, LOGGER.getResourceBundleName());
		//validationErrors.add(validateError);
		//if (validationErrors != null && validationErrors.size() > 0) {
		//	throw new FoundationApplicationException(validationErrors, CLASSNAME, METHODNAME, null);
		//}
		
		// Must always call super() to run the data service layer validation.
		super.validateDelete(noun, nounPart, controlParameters);	
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}

	@Override
	protected void change(Object aNoun, Object aNounPart, List controlParameters) throws AbstractApplicationException {
		// TODO Auto-generated method stub
		
		//part 1 
		ProjectType projectType = (ProjectType) aNounPart;
		String pName = projectType.getProjectIdentifier()
				.getExternalIdentifier().getName();
	    String uniquIdString = projectType.getProjectIdentifier().getUniqueID();
		//part 2
		ProjectUpdateCmdImpl projectUpdateCmd = new ProjectUpdateCmdImpl();
		CommandContext ctx = new CommandContextImpl();
		com.ibm.commerce.datatype.TypedProperty tProperty = new com.ibm.commerce.datatype.TypedProperty();
		tProperty.put("xproject_id", Long.parseLong(uniquIdString));
		tProperty.put("prjname", pName);
		
		ctx.setRequestProperties(tProperty);
		projectUpdateCmd.setCommandContext(ctx);
		//part 3
		try {
			projectUpdateCmd.validateParameters();
			projectUpdateCmd.execute();
		} catch (ECException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}

